@@ -1,4 +1,4 @@
-# $Id: XMLServer.pm,v 1.6 2003/11/17 22:30:12 mjb47 Exp $
+# $Id: XMLServer.pm,v 1.19 2005/11/15 22:03:01 mjb47 Exp $
 
 use strict;
 use warnings;
@@ -7,8 +7,7 @@ use XML::LibXSLT;
 
 package DBIx::XMLServer;
 
-our $VERSION = '0.01';
-our $MAXPAGESIZE = 100;
+our $VERSION = '0.02';
 
 my $our_ns = 'http://boojum.org.uk/NS/XMLServer';
 
@@ -25,7 +24,7 @@ my $sql_ns = sub {
 package DBIx::XMLServer::Field;
 use Carp;
 
-our $VERSION = sprintf '%d.%03d', (q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+our $VERSION = sprintf '%d.%03d', (q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
 
 sub new {
   my $proto = shift;
@@ -101,54 +100,205 @@ sub result {
 
 1;
 
-package DBIx::XMLServer::Request;
-use DBIx::XMLServer::XPathParser;
-use Carp;
+package DBIx::XMLServer::OrderSpec;
 
-our $VERSION = sprintf '%d.%03d', (q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
-
-our $parser = new DBIx::XMLServer::XPathParser;
+our $VERSION = sprintf '%d.%03d', (q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
 
 sub new {
   my $proto = shift;
   my $class = ref($proto) || $proto;
   my $self = {};
-  $self->{XMLServer} = shift 
-    and ref $self->{XMLServer}
-      and $self->{XMLServer}->isa('DBIx::XMLServer') 
-	or croak "No XMLServer object supplied";
 
-  $self->{template} = shift
-    or $self->{template} = $self->{XMLServer}->{template};
+  my ($xmlserver, $node, $dir) = @_;
+  $self->{field} = new DBIx::XMLServer::Field($xmlserver, $node);
+  $self->{dir} = $dir;
+
+  bless($self, $class);
+  return $self;
+}
+
+sub orderspec {
+  my $self = shift;
+  my $spec = $self->{field}->select;
+  for ($self->{dir}) {
+    defined $_ or last;
+    /^ascending$/ && do {
+      $spec .= ' ASC';
+      last;
+    };
+    /^descending$/ && do {
+      $spec .= ' DESC';
+      last;
+    };
+  }
+  return $spec;
+}
+
+1;
+
+package DBIx::XMLServer::Request;
+use Carp;
+use Text::Balanced qw(extract_bracketed);
+
+our $VERSION = sprintf '%d.%03d', (q$Revision: 1.19 $ =~ /(\d+)\.(\d+)/);
+
+if ($ lt v5.7)
+{
+  require Exporter;
+  our @ISA = qw(Exporter);
+  our @EXPORT = qw(IsNCNameStartChar IsNCNameChar);
+}
+
+# Look for an initial segment of the string which looks like an XPath
+# pattern
+sub get_xpath {
+  my $text = shift;
+
+  # Repeatedly skip XPath-like stuff and bracketed things
+  while( (extract_bracketed($text, "[(\"'",
+			    '[-|@_./:[:alnum:][:space:]]*')) [0]) {};
+  # Skip any more XPath-like stuff
+  $text =~ m'\G[-|@_./:[:alnum:][:space:]]*'g;
+
+  return substr($text, 0, pos $text), substr($text, pos $text);
+}
+
+BEGIN {
+  # This hack is because Perl 5.6.1 appears to be buggy and not
+  # allow unicode character properties to be declared in a package
+  # pther than main.
+  our $property_package = $ lt v5.8 ? 'main' : 'DBIx::XMLServer::Request';
+  eval <<END_PROPERTIES;
+  package $property_package;
+
+  # These are the ranges defined by XML 1.1, as these
+  # are more up-to-date w.r.t Unicode those defined by
+  # XML 1.0 (3rd ed).  They're also much simpler to specify.
+  sub IsNCNameStartChar {
+      return <<END;
+41	5A
+5F
+61	7A
+C0	D6
+D8	F6
+F8	2FF
+370	37D
+37F	1FFF
+200C	200D
+2070	218F
+2C00	2FEF
+3001	D7FF
+F900	FDCF
+FDF0	FFFD
+10000	EFFFF
+END
+  }
+
+  sub IsNCNameChar {
+      return <<END;
+2D	2E
+30	39
+41	5A
+5F
+61	7A
+B7
+C0	D6
+D8	F6
+F8	2FF
+300	36F
+370	37D
+37F	1FFF
+200C	200D
+203F	2040
+2070	218F
+2C00	2FEF
+3001	D7FF
+F900	FDCF
+FDF0	FFFD
+10000	EFFFF
+END
+  }
+
+END_PROPERTIES
+}
+
+# Definition of NCName as per XML Namespaces 1.1
+use utf8;
+our $NCName = qr/(?:\p{IsNCNameStartChar}\p{IsNCNameChar}*)/;
+
+sub add_prefix($$) {
+
+  my ( $xpath, $prefix ) = @_;
+
+  while ( $xpath =~ s/
+            ^( (?:[^'"]*(?:"[^"]*"|'[^']*'))*[^'"]*
+               (?: (?<!attribute|namespace)(?<!\s)\s*::\s*
+                 | (?: (?: (?<=[\@([,\/|+-=<>])
+                         | (?<=[<>!]=|\/\/|::) 
+                         | ^) 
+                       (?: $NCName\s+ $NCName\s+ )*
+                     | (?: (?<=[.\])"'])
+                         | [0-9]+(?:\.[0-9]+)?\s
+                         | \$$NCName(?::$NCName)?\s ) 
+                       \s* $NCName\s+
+                       (?: $NCName\s+ $NCName\s+ )* )
+                   (?<![:\$\@]|\p{IsNCNameChar}) ) )
+            ($NCName)
+            (\s+[^:(\s]|\s*(?![:(\s])\P{IsNCNameChar}|$)
+          /$1$prefix:$2$3/x ) {}
+
+  return $xpath;
+}
+
+sub new {
+  my $proto = shift;
+  my $class = ref($proto) || $proto;
+  my $self;
+  if($#_ <= 1) {
+    $self = {};
+    $self->{XMLServer} = shift;
+    $self->{template} = shift;
+  } else {
+    $self = { @_ };
+  };
+  ref $self->{XMLServer}
+    and $self->{XMLServer}->isa('DBIx::XMLServer') 
+      or croak "No XMLServer object supplied";
+
+  $self->{template} or $self->{template} = $self->{XMLServer}->{template};
+  $self->{template}->isa('XML::LibXML::Element')
+    or croak "Template is not a XML::LibXML::Element";
+
+  $self->{template}->localname eq 'template'
+    && $self->{template}->namespaceURI eq $our_ns
+      or croak "Template is not <sql:template>";
   $self->{main_table} = $self->{template}->getAttribute('table')
     or croak "The <sql:template> element has no `table' attribute";
   $self->{ns} = $self->{template}->getAttribute('default-namespace');
   my $p = &$sql_ns($self->{template});
   $self->{record} = $self->{template}->findnodes(".//$p:record/*[1]")->shift
     or croak "The <sql:template> element contains no <sql:record> element";
-  
+
   $self->{criteria} = [];
   $self->{page} = 0;
-  $self->{pagesize} = 100;
+  $self->{pagesize} = $self->{XMLServer}->{maxpagesize}
+    unless defined $self->{pagesize};
+  $self->{rowcount} = $self->{XMLServer}->{rowcount}
+    unless defined $self->{rowcount};
   bless($self, $class);
   return $self;
 }
 
-sub parse {
+sub real_parse {
   my $self = shift;
   my $query = shift or croak "No query string supplied";
-  $::XPATH_DEFAULT_NAMESPACE = $self->{ns};
-  $::XPATH_DEFAULT_NAMESPACE = &$sql_ns($self->{record}, $self->{ns})
-    if defined $self->{ns} && $self->{ns} ne '*';
   foreach(split /&/, $query) {
     # Un-URL-encode the string
-    s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
     tr/+/ /;
-    # Split it into key and condition by removing an initial XPath
-    # expression matching `Pattern'
-    my $r = $parser->Pattern($_)
-      or return "Unrecognised parameter: '$_'\n";
-    my ($key, $condition) = @$r;
+    s/%([0-9A-Fa-f][0-9A-Fa-f])/chr(hex($1))/eg;
+    # Split it into key and condition
+    my ($key, $condition) = get_xpath($_);
+    $key or return "Unrecognised condition: '$condition'";
     for ($key) {
       /^fields$/ && do {
 	$condition =~ s/^=// 
@@ -156,21 +306,52 @@ sub parse {
 	$self->{fields} = $condition;
 	last; 
       };
+      /^order$/ && do {
+	$condition =~ s/^=//
+	  or return "Expected '=' after 'order' but found '$condition'";
+	$self->{order} = $condition;
+	last;
+      };
       /^page$/ && do { # The page number
-        $condition =~ /^=(\d+)$/
+        $condition =~ /^=([1-9]\d*)$/
 	  or return "Unrecognised page number: $condition";
-	$self->{page} = $1;
+	$self->{page} = $1 - 1;
 	last;
       };
       /^pagesize$/ && do { # The page size
         $condition =~ /^=(\d+)$/
 	  or return "Unrecognised page size: $condition";
 	$self->{pagesize} = $1;
-	( $1 > 0 and $1 <= $DBIx::XMLServer::MAXPAGESIZE)
-	  or return "Invalid page size: Must be between 0 " .
-	    "and $DBIx::XMLServer::MAXPAGESIZE";
+	defined($self->{XMLServer}->{maxpagesize})
+	  && $self->{XMLServer}->{maxpagesize} > 0
+          and ( ($1 > 0 && $1 <= $self->{XMLServer}->{maxpagesize})
+	    or return "Invalid page size: Must be between 1 " .
+	      "and $self->{XMLServer}->{maxpagesize}");
 	last;
       };
+      /^format$/ && $self->{userformat} && do {
+          $condition =~ s/^=// 
+            or return "Expected '=' after 'format' but found '$condition'";
+          my $root = $self->{XMLServer}->{doc}->documentElement;
+          my $p = &$sql_ns($root);
+	  $self->{template} = $root->findnodes("/$p:spec/$p:template[@" 
+					       . "id='$condition']")->shift
+	    or return "Invalid format.  Must be one of "
+	      . join(', ', map("'" . $_->value . "'", 
+			       $root->findnodes("/$p:spec/$p:template/@"."id")))
+              . ".";
+	  $self->{template}->localname eq 'template'
+	    && $self->{template}->namespaceURI eq $our_ns
+	    or croak "Template is not <sql:template>";
+	  $self->{main_table} = $self->{template}->getAttribute('table')
+	    or croak "The <sql:template> element has no `table' attribute";
+	  $self->{ns} = $self->{template}->getAttribute('default-namespace');
+	  $p = &$sql_ns($self->{template});
+	  $self->{record} = $self->{template}->findnodes(".//$p:record/*[1]")
+	    ->shift
+	    or croak "The <sql:template> element contains no <sql:record> element";
+	  last;
+	};
       # Anything else we treat as a search criterion
       push @{$self->{criteria}}, [$key, $condition];
     }
@@ -181,9 +362,15 @@ sub parse {
 sub do_criteria {
   my $self = shift;
 
+  my $prefix = $self->{ns};
+  $prefix = &$sql_ns($self->{record}, $self->{ns})
+    if defined $self->{ns} && $self->{ns} ne '*';
   my $p = &$sql_ns($self->{record});
   foreach(@{$self->{criteria}}) {
     my $key = $_->[0];
+    # Fix up a default namespace
+    $key = add_prefix($key, $prefix) if $prefix;
+    # Find the field
     my @nodelist = $self->{record}->findnodes($key);
     my $node;
     if(@nodelist eq 1 && $nodelist[0]->isa('XML::LibXML::Attr')) {
@@ -227,28 +414,23 @@ sub build_output {
   $doc->adoptNode($new_template);
   $doc->setDocumentElement($new_template);
   my $p = &$sql_ns($new_template);
-  my $record;
-  foreach my $node ($new_template->findnodes(".//$p:*")) {
-    for($node->nodeName) {
-      /record/ && do {
-	$record = $node;
-      };
-    }
-  }
-
-  $record or croak "There is no <sql:record> element in the template";
+  my $record = $new_template->findnodes(".//$p:record")->shift
+    or croak "There is no <sql:record> element in the template";
   $self->{newrecord} = $record->findnodes('*')->shift
     or croak "The <sql:record> element has no child element";
 
+  $self->{rowcount} = 'NONE'
+    unless $new_template->findnodes(".//$p:meta[@ type='rows']")->size();
+
   # Find the nodes to return
   if(defined $self->{fields}) {
-    $::XPATH_DEFAULT_NAMESPACE = $self->{ns};
-    $::XPATH_DEFAULT_NAMESPACE = &$sql_ns($self->{newrecord}, $self->{ns})
+    my $prefix = $self->{ns};
+    $prefix = &$sql_ns($self->{newrecord}, $self->{ns})
       if defined $self->{ns} && $self->{ns} ne '*';
-    my $r = $parser->Pattern($self->{fields})
-      or return "Unrecognised fields: '$self->{fields}'";
-    return "Unexpected text: '" . $r->[1] . "'" if $r->[1];
-    $self->{fields} = $r->[0];
+    my ($r, $s) = get_xpath($self->{fields});
+    return "Unexpected text: '$s'" if $s;
+    $r = add_prefix($r, $prefix) if $prefix;
+    $self->{fields} = $r;
   } else {
     $self->{fields} = '.';
   }
@@ -267,6 +449,38 @@ sub build_output {
     }
   }
 
+  # Find the nodes to order by
+  if(defined $self->{order}) {
+    my $prefix = $self->{ns};
+    $prefix = &$sql_ns($self->{newrecord}, $self->{ns})
+      if defined $self->{ns} && $self->{ns} ne '*';
+    my $order = $self->{order};
+    my @order;
+    while ( $order ne '' ) {
+      my ($xpath, $more) = get_xpath($order);
+      $xpath = add_prefix($xpath, $prefix) if $prefix;
+      $xpath =~ s/ +(ascending|descending) *$//;
+      my $dir = $1;
+      my @o = $self->{newrecord}->findnodes($xpath)
+	or return "Invalid field in order clause: $xpath\n";
+      foreach (@o) {
+	my @f = $_->findnodes(
+	  $_->nodeType == XML::LibXML::XML_ATTRIBUTE_NODE
+	    ? "../$p:field[\@attribute='".$_->nodeName."']"
+	    : ".//$p:field" )
+	  or return "No non-static data matched by order clause: $xpath\n";
+	foreach (@f) {
+	  push @order,
+	    new DBIx::XMLServer::OrderSpec($self->{XMLServer}, $_, $dir);
+	}
+      }
+      return "Unexpected order: '$order'" 
+	unless $more eq '' || $more =~ s/^,//;
+      $order = $more;
+    }
+    $self->{order} = \@order;
+  }
+
   # Prune away what we don't want to return
   _prune($self->{newrecord});
 
@@ -318,8 +532,17 @@ sub add_join {
   } unless $self->{joinhash}->{$table};
 }
 
-sub build_joins {
-  my $self = shift;
+sub parse {
+  my ($self, $arg) = @_;
+  my $err;
+
+  $self->{doc} = new XML::LibXML::Document;
+  $self->{arg} = $arg;
+  $err = $self->real_parse($arg) and return $err;
+  $err = $self->do_criteria and return $err;
+  $err = $self->build_output($self->{doc}) and return $err;
+  $err = $self->build_fields and return $err;
+
   $self->{jointext} = [];
   $self->{joinhash} = {};
   $self->add_join($self->{main_table});
@@ -328,40 +551,67 @@ sub build_joins {
       $self->add_join($_);
     }
   }
+
+  my $select;
+  my $from;
+  my $where;
+  my $order;
+  my $limit;
+
+  eval {
+    $where = join(' AND ', map($_->[0]->where($_->[1]),
+				      @{$self->{criteria}})) || '1';
+    $from = join(' ', @{$self->{jointext}});
+  };
+  return $@ if $@;
+
+  $self->{count_query} = "SELECT COUNT(*) FROM $from WHERE $where";
+
   foreach my $f (@{$self->{fields}}) {
     foreach ($f->join) {
       $self->add_join($_);
     }
   }
-  return undef;
-} 
 
-# Process a request
-# $results = $xmlout->process($query_string);
-sub process {
-  my ($self, $arg) = @_;
-  my $err;
+  foreach my $o (@{$self->{order}}) {
+    foreach ($o->{field}->join) {
+      $self->add_join($_);
+    }
+  }
 
-  my $doc = new XML::LibXML::Document;
-
-  $err = $self->parse($arg) and return $err;
-  $err = $self->do_criteria and return $err;
-  $err = $self->build_output($doc) and return $err;
-  $err = $self->build_fields and return $err;
-  $err = $self->build_joins and return $err;
-
-  my $query;
   eval {
-    my $select = join(',', map($_->select, @{$self->{fields}})) || '0';
-    my $from = join(' ', @{$self->{jointext}});
-    my $where = join(' AND ', map($_->[0]->where($_->[1]),
-				      @{$self->{criteria}})) || '1';
-    my $limit = ($self->{page} * $self->{pagesize}) . ", $self->{pagesize}";
-    $query = "SELECT $select FROM $from WHERE $where LIMIT $limit";
+    $select = join(',', map($_->select, @{$self->{fields}})) || '0';
+    $order = (defined $self->{order} && scalar @{$self->{order}}) ? 
+      ' ORDER BY ' . join(',', map($_->orderspec, @{$self->{order}})) 
+        : '';
+    $limit = ($self->{pagesize} > 0) ? 
+      ' LIMIT ' . ($self->{page} * $self->{pagesize}) . ", $self->{pagesize}"
+	: '';
+    $from = join(' ', @{$self->{jointext}});
   };
   return $@ if $@;
 
+  $self->{query} = "SELECT $select FROM $from WHERE $where$order$limit";
+
+  return undef;
+}
+
+# Process a request
+# $results = $xmlout->process();
+sub process {
+  my $self = shift;
+  my %args = @_;
+  my $err;
+
+  $self->{query} 
+    or croak "DBIx::XMLServer::Request: must call parse before process";
+
+  $args{rowcount} = $self->{rowcount} unless $args{rowcount};
+
   # Do the query
+  my $query = $self->{query};
+  $query =~ s/^SELECT/SELECT SQL_CALC_FOUND_ROWS/ 
+    if $args{rowcount} eq 'FOUND_ROWS';
   my $sth = $self->{XMLServer}->{dbh}->prepare($query);
   $sth->execute or croak $sth->errstr;
 
@@ -369,10 +619,10 @@ sub process {
   my $r = $self->{newrecord}->parentNode;
   my @row;
   while(@row = $sth->fetchrow_array) {
-    
+
     # Clone the template record and insert after the previous record
     $r = $r->parentNode->insertAfter($self->{newrecord}->cloneNode(1), $r);
-    
+
     # Fill in the values
     my $p = &$sql_ns($self->{newrecord});
     my @n = $r->findnodes(".//$p:field");
@@ -383,8 +633,31 @@ sub process {
 
   }
 
+  my $rows = 0;
+  do {
+    my @r;
+    @r = $self->{XMLServer}->{dbh}->selectrow_array('SELECT FOUND_ROWS()')
+      or croak $self->{XMLServer}->{dbh}->errstr;
+    $rows = $r[0];
+  } if $args{rowcount} eq 'FOUND_ROWS';
+  do {
+    my @r;
+    @r = $self->{XMLServer}->{dbh}->selectrow_array($self->{count_query})
+      or croak $self->{XMLServer}->{dbh}->errstr;
+    $rows = $r[0];
+  } if $args{rowcount} eq 'COUNT';
+
+  my %params = (
+    'args' => $self->{arg},
+    'page' => $self->{page},
+    'pagesize' => $self->{pagesize},
+    'query' => $self->{query},
+    'rows' => $rows,
+  );		
+
   # Process through XSLT to produce the result
-  return $self->{XMLServer}->{xslt}->transform($doc);
+  return $self->{XMLServer}->{xslt}->transform($self->{doc}, 
+    XML::LibXSLT::xpath_to_string(%params));
 }
 
 1; 
@@ -441,20 +714,34 @@ EOF
 sub new {
   my $proto = shift;
   my $class = ref($proto) || $proto;
-  my $self = {};
+  my $self;
+  my $doc;
+
+  # Deal with the parameters
+  if(ref $_[0]) { # dbh, doc [, template]
+    $self = {};
+    $self->{dbh} = shift or croak "No database handle supplied";
+    $doc = shift or croak "No template file supplied";
+    $self->{template} = shift;
+  } else { # Named parameters
+    $self = { @_ };
+    $self->{dbh} or croak "No database handle supplied";
+    $doc = $self->{doc} or croak "No template file supplied";
+  }
   bless($self, $class);
 
   my $parser = new XML::LibXML;
-
-  $self->{dbh} = shift or croak "No database handle supplied";
-  my $doc = shift or croak "No template file supplied";
-  $self->{template} = shift;
-
   ref $doc or $doc = $parser->parse_file($doc)
     or croak "Couldn't parse template file '$doc'";
+  $doc->isa('XML::LibXML::Document')
+    or croak "This isn't a XML::LibXML::Document";
   $self->{doc} = $doc;
 
-  my $p = &$sql_ns($doc->documentElement);
+  my $root = $doc->documentElement;
+  $root->localname eq 'spec' && $root->namespaceURI eq $our_ns
+    or croak "Document element is not <sql:spec>";
+
+  my $p = &$sql_ns($root);
 
   # Find all the field type definitions and parse them
   $self->{types} = {};
@@ -468,6 +755,13 @@ sub new {
         ->shift
       or croak "No <sql:template> element found";
 
+  $self->{template}->isa('XML::LibXML::Element')
+    or croak "Template is not a XML::LibXML::Element";
+
+  $self->{template}->localname eq 'template'
+    && $self->{template}->namespaceURI eq $our_ns
+      or croak "Template is not <sql:template>";
+
   # Parse our XSLT stylesheet
   my $xslt = new XML::LibXSLT;
   my $f = $INC{'DBIx/XMLServer.pm'};
@@ -477,14 +771,29 @@ sub new {
   $self->{xslt} = $xslt->parse_stylesheet($style_doc)
     or croak "Error parsing stylesheet '$f'";
 
+  $self->{maxpagesize} = 0 unless $self->{maxpagesize};
+  $self->{rowcount} = 'NONE' unless defined $self->{rowcount};
+
   return $self;
 }
 
 sub process {
   my $self = shift;
-  my $query = shift;
-  my $request = new DBIx::XMLServer::Request($self, @_);
-  return $request->process($query);
+  my %args;
+  my $err;
+
+  # Process arguments
+  if($#_ <= 1 && $_[0] ne 'query') {
+    $args{query} = shift
+      or croak "No query string given";
+  } else { # Named parameters
+    %args = @_;
+  }
+
+  $args{XMLServer} = $self;
+  my $request = new DBIx::XMLServer::Request(%args);
+  $err = $request->parse($args{query}) and return $err;
+  return $request->process();
 }
 
 1;
@@ -517,6 +826,9 @@ into XML, but also to parse the query string.  To the user, the format
 of the query string is very natural in relation to the XML data which
 they will receive.
 
+All the methods of this object can take a hash of named parameters instead
+of a list of parameters.
+
 One C<DBIx::XMLServer> object can process several queries.  The
 following steps take place in processing a query:
 
@@ -546,8 +858,13 @@ template, and returned to the caller.
   my $xml_server = new DBIx::XMLServer( $dbh, $template_doc 
                                         [, $template_node] );
 
+  my $xml_server = new DBIx::XMLServer( dbh => $dbh,
+                                        doc => $template_doc,
+                                        template => $template_node,
+                                        maxpagesize => $maxpagesize );
+
 The constructor for C<DBIx::XMLServer> takes two mandatory arguments
-and one optional argument.
+and two optional arguments.
 
 =over
 
@@ -568,15 +885,28 @@ this argument may be used to pass an C<XML::LibXML::Element> object
 indicating which template should be used.  By default the first
 template in the file is used.
 
+=item C<$maxpagesize>
+
+This option may be used to limit the number of records than will be
+returned in a query.  The user can choose a page size smaller than
+this by using the C<pagesize> option on their query (see below), but 
+they will not be allowed to request a page size larger than this 
+maximum.
+
 =back
 
 =head2 process()
 
   my $result = $xml_server->process( $query [, $template_node] );
 
+  my $result = $xml_server->process( query => $query,
+                                     template => $template_node,
+                                     rowcount => $rowcount,
+                                     userformat => $userformat );
+
 This method processes an HTTP query and returns an XML document
-containing the results of the query.  There is one mandatory argument
-and one optional argument.
+containing the results of the query.  There are one mandatory argument
+and two optional arguments.
 
 =over
 
@@ -589,6 +919,40 @@ This is the HTTP GET query string to be processed.
 As above, this may indicate which of several templates is to be used
 for this query.  It is an C<XML::LibXML::Element> object.
 
+=item C<$rowcount>
+
+It is possible to limit the number of rows returned in one query, either
+in response to a user request (by using the C<pagesize> option, see below)
+or by passing the C<maxpagesize> option when creating the C<DBIx::XMLServer>
+object.  In these cases it may be useful to know the total number of rows 
+that would have been returned had no limit been in place.  The number of
+rows can be put into the output XML document using the
+B<< <sql:meta type="rows"> >> element in the template (see below).  This
+argument chooses how this information should be obtained from the database.
+
+=over
+
+=item FOUND_ROWS
+
+Passing C<< rowcount => 'FOUND_ROWS' >> tells the module to use the
+B<SQL_COUNT_FOUND_ROWS> option and the B<FOUND_ROWS> function.  If
+your database supports these, use this option.
+
+=item COUNT
+
+Passing C<< rowcount => 'COUNT' >> means that a second query will be
+done after the main database query, of this form:
+
+  SELECT COUNT(*) FROM ... WHERE ...
+
+=back
+
+=item C<$userformat>
+
+Setting this to a true value allows the user to choose between several
+templates in the file by specifying the C<format> option in the query
+string.
+
 =back
 
 The return value of this method is either an C<XML::LibXML::Document>
@@ -716,7 +1080,34 @@ this option is again an XPath expression which selects a part of the
 record template to be returned.
 
 Other special options control how many records are returned on each
-page and which page of the results should be returned.
+page, which page of the results should be returned, and may choose one
+of several templates in the file.
+
+The template to use for a query is chosen as follows:
+
+=over
+
+=item 1.
+
+If the B<userformat> option is set when calling C<DBIx::XMLServer::process()>
+and the user has chosen a template with the B<format> option in the query
+string, that template is used.
+
+=item 2.
+
+Otherwise, if a template was specified when calling 
+C<DBIx::XMLServer::process()>, then that template is used.
+
+=item 3.
+
+Otherwise, if a template was specified when constructing the 
+C<DBIx::XMLServer> object, then that template is used.
+
+=item 4.
+
+Otherwise, the first template in the file is used.
+
+=back
 
 =head1 THE TEMPLATE FILE
 
@@ -909,6 +1300,10 @@ output document.  It also contains one B<< <sql:record> >> element,
 which defines that part of the output document which is repeated for
 each row returned from the SQL query.
 
+Further, the template element may contain B<< <sql:meta> >> elements
+which indicate that certain information about the query should be
+inserted into the output document.
+
 As the output document is formed from the content of the B<<
 <sql:template> >> element, it follows that this element must have
 exactly one child element.
@@ -917,6 +1312,12 @@ The B<< <sql:template> >> may have the following attributes:
 
 =over
 
+=item attribute: B<id>
+
+This optional attribute gives a unique identifier to the template.  The
+user may, if allowed, choose which template to use by specifying the
+B<format> option in the query string together with this identifier.
+
 =item attribute: B<table>
 
 This mandatory attribute specifies the main table for this template, to
@@ -1016,6 +1417,59 @@ The parent element has the B<xsi:nil> attribute set.
 
 =back
 
+=head2 The sql:omit attribute
+
+Any element in the record may have the Boolean attribute B<sql:omit>.
+If this attribute is set, then this element will be omitted from any
+record in which the element is empty (because child elements have been
+omitted).
+
+=head2 The meta element
+
+The B<< <sql:meta> >> element is used for putting information about the
+query into the output document.  The information is selected by the
+B<type> attribute of the element.  The following B<type> attributes
+are recognised:
+
+=over
+
+=item type='args'
+
+This gives the original query string passed to B<DBIx::XMLServer>.
+
+=item type='page'
+
+This gives the page number within the results, as selected by the
+B<page=> option in the query string.
+
+=item type='pagesize'
+
+This gives the page size, as selected by the B<pagesize=> option in 
+the query string.
+
+=item type='query'
+
+This gives the SQL query which was executed to produce the results.
+
+=item type='rows'
+
+This gives the number of rows which the query would have returned, had
+it not been for the B<page=> and B<pagesize=> options.  To tell the
+module how to find this information, set the B<rowcount> option when
+processing the request (see above).
+
+=back
+
+The B<< <sql:meta> >> element in the template will be replaced by the
+corresponding string in the output document.  Alternatively, it is
+possible to place the string into the output document as an attribute
+to the parent element of the B<< <sql:meta> >> element.  To do this,
+include an attribute B<attribute="name"> on the B<< <sql:meta> >> element,
+where B<name> is the local name of the attribute.  To add a namespace
+to the attribute, additionally include an attribute B<namespace="foo">
+on the B<< <sql:meta> >> element, replacing B<foo> with whatever
+namespace should be used.
+
 =head1 SPECIAL OPTIONS IN THE QUERY STRING
 
 The HTTP query string may contain certain special options which are
@@ -1038,7 +1492,21 @@ expression.
 =item pagesize = <number>, page = <number>
 
 These options give control over how many records are returned in one
-query, and which of several pages is returned.
+query, and which of several pages is returned.  To put a limit on the
+page size which can be requested, use the B<maxpagesize> option when
+creating the C<DBIx::XMLServer> object.  By default there is no limit.
+
+=item order = <list>
+
+This option controls how records are ordered in the output document.
+The B<< <list> >> should be a comma-separated list of XPath
+expressions, each optionally followed by a space and the string
+B<ascending> or B<descending>.  Each of these XPath expressions is
+evaluated within the context of the single child of the 
+B<< <sql:record> >> element and should select one or more fields;
+these fields are used to order the result records.  Fields are used in
+the order that they appear in the list; if a single list element
+selects more than one field, they are used in document order.
 
 =back
 
@@ -1112,7 +1580,19 @@ up before being returned to the caller.
 
 There are quite a lot of stray namespace declarations in the output.
 They make no difference to the semantic meaning of the markup, but
-they are ugly.
+they are ugly.  I gather that XML::LibXML will provide the means to
+remove them in the near future.
+
+The way we build JOIN expressions isn't very clever, and probably
+doesn't work for anything more than the simplest situations.
+
+The way we add a default prefix to every XPath expression is a bit of
+a hack.  I think the only way to fix this is to wait for XPath 2.0 to
+be widely available.
+
+This module has been written to use MySQL and has only been tested on
+that platform.  I would be interested to hear from users who have been
+able to make it work on other platforms.
 
 =head1 SEE ALSO
 
@@ -1124,7 +1604,7 @@ Martin Bright E<lt>martin@boojum.org.ukE<gt>
 
 =head1 COPYRIGHT AND LICENCE
 
-Copyright (C) 2003 Martin Bright
+Copyright (C) 2003-4 Martin Bright
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
